-------------------------------------------------------------------------------
-- Append Code: default globals.static_data.mobile_effects.players.craftitem
-------------------------------------------------------------------------------
require 'default:globals.static_data.mobile_effects.players.craftitem'
-------------------------------------------------------------------------------
--[[							MOD INFORMATION
	---------------------------------------------------------------------------							  		
	In order to add a leveling system we need to capture the instance where
	the player crafts the item. Once it is crafted we can check the recipe
	table for the crafted item. If the recipe table contains a key called
	CanLevelUp = true (defaults to false). Then you can write to a table 
	which is located on the player called CraftingExperience. The items
	template_id is saved as a key which contains a Level and CurrentExperience
	key value pairs. Every time a player crafts these items, it adds +1 until
	the reach the experience cap for that level, they level +1 then the
	CurrentExperience value goes to 0. 

	We need a master table to check the ExperienceRequirements for each level
	so that you can set the threshold to craft to reach the next level. Once
	a player crafts to max level, they will be alerted by a MaxLevelMessage
	which we can customize.
	---------------------------------------------------------------------------
--]]
-------------------------------------------------------------------------------
-- CraftedExperienceTable master table
-------------------------------------------------------------------------------
CraftedExperienceTable = {}

-------------------------------------------------------------------------------
-- MaxLevelMessage
-------------------------------------------------------------------------------
-- when the player hits max level display this event message 
-- %s is replaced by the max level the player can reach
CraftedExperienceTable.MaxLevelMessage = "Masterfully done!! Level %s achieved!!"

-------------------------------------------------------------------------------
-- ExperienceRequirements table
-------------------------------------------------------------------------------
-- the number of items they have to craft to progress
-- the current experience is reset to 0 at each level upgrade.
CraftedExperienceTable.ExperienceRequirements = 
{
	Level1 = 50,
	Level2 = 75,
	Level3 = 100,
	Level4 = 150,
	Level5 = 200,
	Level6 = 250,
	Level7 = 400,
	Level8 = 550,
	Level9 = 700,
	Level10 = 1000,
	Level11 = 1200,
	Level12 = 1400,
	Level13 = 1600,
	Level14 = 2000,
	Level15 = 2400,
	Level16 = 2800,
	Level17 = 3600,
	Level18 = 4400,
	Level19 = 5200,
	Level20 = 6800,
}

-------------------------------------------------------------------------------
-- GetLevel function
-------------------------------------------------------------------------------
-- returns the level of the crafted items template
--@param objectReference (GameObject) - the item that was just crafted
CraftedExperienceTable.GetLevel = function(player, objectReference)
	local craftingExperienceTable = player:GetObjVar("CraftingExperience") or {}
	local objectTemplate = objectReference:GetCreationTemplateId()
	if(craftingExperienceTable[objectTemplate]) then
		return craftingExperienceTable[objectTemplate].Level
	else
		return 0
	end
end

-------------------------------------------------------------------------------
-- HandleItemCreation function
-------------------------------------------------------------------------------
--- handles the leveling
CraftedExperienceTable.HandleItemCreation = function(self, root, success, objectReference, wasStacked, stackCount)
	 if(success == false) then return end
	 local craftingExperienceTable = self.ParentObj:GetObjVar("CraftingExperience") or {}
	 local objectTemplate = objectReference:GetCreationTemplateId()
	 -- do we have a table? if so check for leveling up
	 -- otherwise create the starting table
	 if(craftingExperienceTable[objectTemplate] ~= nil) then
	 	local level = craftingExperienceTable[objectTemplate].Level
	 	local maxLevel = #CraftedExperienceTable.ExperienceRequirements
		local currentExperience = craftingExperienceTable[objectTemplate].CurrentExperience	 	
	 	-- if we maxed out the level then return
	 	if(level > maxLevel-1) then return end
	 	-- progress experience
	 	currentExperience = currentExperience + 1
	 	-- did we reach the experience threashold?
	 	if(currentExperience >= CraftedExperienceTable.ExperienceRequirements["Level"..level]) then
	 		level = level + 1
	 		currentExperience = 0
	 		if(level >= maxLevel) then
	 			self.ParentObj:SystemMessage(string.format(CraftedExperienceTable.MaxLevelMessage,maxLevel),"event")
	 		else
 	 			self.ParentObj:SystemMessage("Crafting " .. objectReference:GetName() .. " - Rank " .. level .. " achieved","event")
	 		else
	 			--??
	 		end
	 	end
	 	craftingExperienceTable[objectTemplate].Level = level
	 	craftingExperienceTable[objectTemplate].CurrentExperience = currentExperience
	 	self.ParentObj:SetObjVar("CraftingExperience",craftingExperienceTable)
	 else
	 	craftingExperienceTable[objectTemplate] = {Level = 1,CurrentExperience = 1,}
	 	self.ParentObj:SetObjVar("CraftingExperience",craftingExperienceTable)
	 end
end

-------------------------------------------------------------------------------
-- Preserve Override of MobileEffectLibrary.CraftItem.HandleItemCreation
-------------------------------------------------------------------------------
-- since we do not want to override how citadel handles item creation
-- we only want to capture the instance when the crafted item is created
-- to add in our leveling system for crafting
MobileEffectLibrary.CraftItem.BaseHandleItemCreation = MobileEffectLibrary.CraftItem.HandleItemCreation

-------------------------------------------------------------------------------
-- Override MobileEffectLibrary.CraftItem.HandleItemCreation
-------------------------------------------------------------------------------
-- override the base HandleItemCreation function so we can do our leveling/experience checks
MobileEffectLibrary.CraftItem.HandleItemCreation = function (self, root, success, objRef, wasStacked, stackCount)
	 -- crafted experience crafted item event handling
	 CraftedExperienceTable.HandleItemCreation(self, root, success, objRef, wasStacked, stackCount)
	 -- do citadels normal crafted item event handling
	 self.BaseHandleItemCreation(self, root, success,objRef,wasStacked,stackCount)
end